@@ -39,7 +39,8 @@ module Data.Equivalence.Monad
      ) where
 
 import Data.Equivalence.STT hiding (equate, equateAll, equivalent, classDesc, removeClass,
-                                    getClass , combine, combineAll, same , desc , remove )
+                                    getClass , combine, combineAll, same , desc , remove,
+                                    values , classes )
 import qualified Data.Equivalence.STT  as S
 
 
@@ -204,6 +205,20 @@ class (Monad m, Applicative m, Ord v) => MonadEquiv c v d m | m -> v, m -> c, m
 
     remove :: c -> m Bool
 
+    {-| This function returns all values represented by
+       some equivalence class.
+
+       @since 0.4.1 -}
+
+    values :: m [v]
+
+    {-| This function returns the list of
+       all equivalence classes.
+
+       @since 0.4.1 -}
+
+    classes :: m [c]
+
     -- Default implementations for lifting via a monad transformer.
     -- Unfortunately, GHC does not permit us to give these also to
     -- 'equate' and 'combine', which already have a default implementation.
@@ -235,6 +250,12 @@ class (Monad m, Applicative m, Ord v) => MonadEquiv c v d m | m -> v, m -> c, m
     default remove      :: (MonadEquiv c v d n, MonadTrans t, t n ~ m) => c -> m Bool
     remove               = lift . remove
 
+    default values      :: (MonadEquiv c v d n, MonadTrans t, t n ~ m) => m [v]
+    values               = lift values
+
+    default classes     :: (MonadEquiv c v d n, MonadTrans t, t n ~ m) => m [c]
+    classes              = lift classes
+
 
 instance (Monad m, Applicative m, Ord v) => MonadEquiv (Class s d v) v d (EquivT s d v m) where
     equivalent x y = EquivT $ do
@@ -281,6 +302,14 @@ instance (Monad m, Applicative m, Ord v) => MonadEquiv (Class s d v) v d (EquivT
       part <- ask
       lift $ S.remove part x
 
+    values = EquivT $ do
+      part <- ask
+      lift $ S.values part
+
+    classes = EquivT $ do
+      part <- ask
+      lift $ S.classes part
+
 instance (MonadEquiv c v d m, Monoid w) => MonadEquiv c v d (WriterT w m) where
     equate  x y = lift $ equate x y
     combine x y = lift $ combine x y
@@ -289,6 +318,7 @@ instance (MonadEquiv c v d m) => MonadEquiv c v d (ExceptT e m) where
     equate  x y = lift $ equate x y
     combine x y = lift $ combine x y
 
+
 instance (MonadEquiv c v d m) => MonadEquiv c v d (StateT s m) where
     equate  x y = lift $ equate x y
     combine x y = lift $ combine x y
