@@ -1,24 +1,23 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE RankNTypes, TemplateHaskell #-}
+{-# LANGUAGE ScopedTypeVariables #-}
 
 module Data.Equivalence.Monad_Test where
 
-import Test.QuickCheck hiding ((===))
+import Test.QuickCheck hiding ((===), classes)
 
 import Data.Equivalence.Monad
 
 import Control.Monad
+import Data.Function (on)
 import Data.Set (Set)
 import qualified Data.Set as Set
 import System.Exit
 
-
-
 --------------------------------------------------------------------------------
 -- Test Suits
 --------------------------------------------------------------------------------
 
-
 -- run :: (Ord a) => STT s Identity (Equiv s (Set a) a)
 run :: (Ord v) => (forall s. EquivM s (Set v) v a) -> a
 run = runEquivM Set.singleton Set.union
@@ -30,6 +29,12 @@ allM f l = liftM and $ mapM f l
 
 getClasses l1 = mapM getClass l1
 
+infixr 9 <.>
+
+-- | Composition: pure function after functorial (monadic) function.
+(<.>) :: Functor m => (b -> c) -> (a -> m b) -> a -> m c
+(f <.> g) a = f <$> g a
+
 
 --------------------------------------------------------------------------------
 -- Properties
@@ -43,7 +48,7 @@ prop_equateAll l' v = runInt $ do
   let l = v:l'
   equateAll l
   d <- classDesc v
-  return  (d == Set.fromList l)
+  return (d == Set.fromList l)
 
 prop_combineAll l' v = runInt $ do
   let l = v:l'
@@ -134,28 +139,49 @@ prop_remove' x y l1' l2' = runInt $ do
   return (Set.fromList l2 == d)
 
 
-prop_classes l1 l1' l2 x y = putStrLn (show el ++ ";" ++ show cl) `whenFail` (el == cl)
-    where l3 = concat (l2 : l1)
-          el = runInt $ do
-                 mapM equateAll l1
-                 mapM removeClass l2
-                 mapM equateAll (l1' :: [[Int]])
-                 res <- mapM classDesc l3
-                 eq <- equivalent x y
-                 return (res,eq)
-          cl = runInt $ do
-                 cls1 <- mapM getClasses l1
-                 mapM combineAll cls1
-                 cls2 <- getClasses l2
-                 mapM remove cls2
-                 cls1' <- mapM getClasses l1'
-                 mapM combineAll cls1'
-                 cls3 <- getClasses l3
-                 res <- mapM desc cls3
-                 [cx,cy] <- getClasses [x,y]
-                 eq <- cx === cy
-                 return (res,eq)
+prop_getClasses l1 l1' l2 x y =
+  putStrLn (show el ++ ";" ++ show cl) `whenFail` (el == cl)
+  where
+    l3 = concat (l2 : l1)
 
+    el = runInt $ do
+      mapM equateAll l1
+      mapM removeClass l2
+      mapM equateAll (l1' :: [[Int]])
+      res <- mapM classDesc l3
+      eq <- equivalent x y
+      return (res,eq)
+
+    cl = runInt $ do
+      cls1 <- mapM getClasses l1
+      mapM combineAll cls1
+      cls2 <- getClasses l2
+      mapM remove cls2
+      cls1' <- mapM getClasses l1'
+      mapM combineAll cls1'
+      cls3 <- getClasses l3
+      res <- mapM desc cls3
+      [cx,cy] <- getClasses [x,y]
+      eq <- cx === cy
+      return (res,eq)
+
+prop_values l = runInt $ do
+    mapM (\x -> equate x x) l
+    sameSet l <$> values
+  where
+    sameSet = (==) `on` Set.fromList
+
+prop_classes l = runInt $ do
+    mapM equateAll (l :: [[Int]])
+    classes1 <- uniqClass =<< mapM getClass =<< values
+    sameClasses classes1 =<< classes
+  where
+    uniqClass []     = return []
+    uniqClass (c:cs) = (c :) <$> do
+      uniqClass =<< filterM (not <.> (c ===)) cs
+
+    sameClasses []       cs2 = return $ null cs2
+    sameClasses (c:cs1') cs2 = sameClasses cs1' =<< filterM (not <.> (c ===)) cs2
 
 return []
 
