@@ -1,18 +1,13 @@
 module Main where
 
-import Test.Framework
 import qualified Data.Equivalence.Monad_Test
 
 --------------------------------------------------------------------------------
 -- Test Suits
 --------------------------------------------------------------------------------
 
-main = defaultMain [tests]
-
-tests = testGroup "Data" [
-         Data.Equivalence.Monad_Test.tests
-       ]
+main = Data.Equivalence.Monad_Test.main
 
 --------------------------------------------------------------------------------
 -- Properties
---------------------------------------------------------------------------------
\ No newline at end of file
+--------------------------------------------------------------------------------
